@@ -29,6 +29,13 @@ DOT_ATOM_TEXT_UTF8 = '[' + ATEXT_UTF8 + ']+(?:\\.[' + ATEXT_UTF8 + ']+)*'
 # the beginning or end of a *dot-atom component* of a hostname either.
 ATEXT_HOSTNAME = r'(?:(?:[a-zA-Z0-9][a-zA-Z0-9\-]*)?[a-zA-Z0-9])'
 
+# Length constants
+# RFC 3696 + errata 1003 + errata 1690 (https://www.rfc-editor.org/errata_search.php?rfc=3696&eid=1690)
+# explains the maximum length of an email address is 254 octets.
+EMAIL_MAX_LENGTH = 254
+LOCAL_PART_MAX_LENGTH = 64
+DOMAIN_MAX_LENGTH = 255
+
 # ease compatibility in type checking
 if sys.version_info >= (3,):
     unicode_class = str
@@ -135,14 +142,18 @@ class ValidatedEmail(object):
 
     """Tests use this."""
     def __eq__(self, other):
-        if self.email == other.email and self.local_part == other.local_part and self.domain == other.domain \
-           and self.ascii_email == other.ascii_email and self.ascii_local_part == other.ascii_local_part \
-           and self.ascii_domain == other.ascii_domain \
-           and self.smtputf8 == other.smtputf8 \
-           and repr(sorted(self.mx) if self.mx else self.mx) == repr(sorted(other.mx) if other.mx else other.mx) \
-           and self.mx_fallback_type == other.mx_fallback_type:
-            return True
-        return False
+        return (
+            self.email == other.email
+            and self.local_part == other.local_part
+            and self.domain == other.domain
+            and self.ascii_email == other.ascii_email
+            and self.ascii_local_part == other.ascii_local_part
+            and self.ascii_domain == other.ascii_domain
+            and self.smtputf8 == other.smtputf8
+            and repr(sorted(self.mx) if self.mx else self.mx)
+            == repr(sorted(other.mx) if other.mx else other.mx)
+            and self.mx_fallback_type == other.mx_fallback_type
+        )
 
     """This helps producing the README."""
     def as_constructor(self):
@@ -156,6 +167,18 @@ class ValidatedEmail(object):
                        ) \
             + ")"
 
+    """Convenience method for accessing ValidatedEmail as a dict"""
+    def as_dict(self):
+        return self.__dict__
+
+
+def __get_length_reason(addr, utf8=False, limit=EMAIL_MAX_LENGTH):
+    diff = len(addr) - limit
+    reason = "({}{} character{} too many)"
+    prefix = "at least " if utf8 else ""
+    suffix = "s" if diff > 1 else ""
+    return reason.format(prefix, diff, suffix)
+
 
 def validate_email(
     email,
@@ -208,9 +231,6 @@ def validate_email(
     if not ret.smtputf8:
         ret.ascii_email = ret.ascii_local_part + "@" + ret.ascii_domain
 
-    # RFC 3696 + errata 1003 + errata 1690 (https://www.rfc-editor.org/errata_search.php?rfc=3696&eid=1690)
-    # explains the maximum length of an email address is 254 octets.
-    #
     # If the email address has an ASCII representation, then we assume it may be
     # transmitted in ASCII (we can't assume SMTPUTF8 will be used on all hops to
     # the destination) and the length limit applies to ASCII characters (which is
@@ -231,33 +251,24 @@ def validate_email(
     # longer than the number of characters.
     #
     # See the length checks on the local part and the domain.
-    if ret.ascii_email and len(ret.ascii_email) > 254:
+    if ret.ascii_email and len(ret.ascii_email) > EMAIL_MAX_LENGTH:
         if ret.ascii_email == ret.email:
-            reason = " ({} character{} too many)".format(
-                len(ret.ascii_email) - 254,
-                "s" if (len(ret.ascii_email) - 254 != 1) else ""
-            )
-        elif len(ret.email) > 254:
+            reason = __get_length_reason(ret.ascii_email)
+        elif len(ret.email) > EMAIL_MAX_LENGTH:
             # If there are more than 254 characters, then the ASCII
             # form is definitely going to be too long.
-            reason = " (at least {} character{} too many)".format(
-                len(ret.email) - 254,
-                "s" if (len(ret.email) - 254 != 1) else ""
-            )
+            reason = __get_length_reason(ret.email, utf8=True)
         else:
-            reason = " (when converted to IDNA ASCII)"
-        raise EmailSyntaxError("The email address is too long{}.".format(reason))
-    if len(ret.email.encode("utf8")) > 254:
-        if len(ret.email) > 254:
+            reason = "(when converted to IDNA ASCII)"
+        raise EmailSyntaxError("The email address is too long {}.".format(reason))
+    if len(ret.email.encode("utf8")) > EMAIL_MAX_LENGTH:
+        if len(ret.email) > EMAIL_MAX_LENGTH:
             # If there are more than 254 characters, then the UTF-8
             # encoding is definitely going to be too long.
-            reason = " (at least {} character{} too many)".format(
-                len(ret.email) - 254,
-                "s" if (len(ret.email) - 254 != 1) else ""
-            )
+            reason = __get_length_reason(ret.email, utf8=True)
         else:
-            reason = " (when encoded in bytes)"
-        raise EmailSyntaxError("The email address is too long{}.".format(reason))
+            reason = "(when encoded in bytes)"
+        raise EmailSyntaxError("The email address is too long {}.".format(reason))
 
     if check_deliverability:
         # Validate the email address's deliverability and update the
@@ -291,11 +302,9 @@ def validate_email_local_part(local, allow_smtputf8=True, allow_empty_local=Fals
     # internationalized, then the UTF-8 encoding may be longer, but
     # that may not be relevant. We will check the total address length
     # instead.
-    if len(local) > 64:
-        raise EmailSyntaxError("The email address is too long before the @-sign ({} character{} too many).".format(
-            len(local) - 64,
-            "s" if (len(local) - 64 != 1) else ""
-        ))
+    if len(local) > LOCAL_PART_MAX_LENGTH:
+        reason = __get_length_reason(local, limit=LOCAL_PART_MAX_LENGTH)
+        raise EmailSyntaxError("The email address is too long before the @-sign {}.".format(reason))
 
     # Check the local part against the regular expression for the older ASCII requirements.
     m = re.match(DOT_ATOM_TEXT + "\\Z", local)
@@ -400,7 +409,7 @@ def validate_email_domain_part(domain):
     # on the assumption that the domain may be transmitted without SMTPUTF8
     # as IDNA ASCII. This is also checked by idna.encode, so this exception
     # is never reached.
-    if len(ascii_domain) > 255:
+    if len(ascii_domain) > DOMAIN_MAX_LENGTH:
         raise EmailSyntaxError("The email address is too long after the @-sign.")
 
     # A "dot atom text", per RFC 2822 3.2.4, but using the restricted
@@ -438,8 +447,20 @@ def validate_email_deliverability(domain, domain_i18n, timeout=DEFAULT_TIMEOUT):
     # Check that the domain resolves to an MX record. If there is no MX record,
     # try an A or AAAA record which is a deprecated fallback for deliverability.
 
-    # Add a trailing period to ensure the domain name is treated as fully qualified.
-    domain += '.'
+    def dns_resolver_resolve_shim(resolver, domain, record):
+        try:
+            # dns.resolver.Resolver.resolve is new to dnspython 2.x.
+            # https://dnspython.readthedocs.io/en/latest/resolver-class.html#dns.resolver.Resolver.resolve
+            return resolver.resolve(domain, record)
+        except AttributeError:
+            # dnspython 2.x is only available in Python 3.6 and later. For earlier versions
+            # of Python, we maintain compatibility with dnspython 1.x which has a
+            # dnspython.resolver.Resolver.query method instead. The only difference is that
+            # query may treat the domain as relative and use the system's search domains,
+            # which we prevent by adding a "." to the domain name to make it absolute.
+            # dns.resolver.Resolver.query is deprecated in dnspython version 2.x.
+            # https://dnspython.readthedocs.io/en/latest/resolver-class.html#dns.resolver.Resolver.query
+            return resolver.query(domain + ".", record)
 
     try:
         # We need a way to check how timeouts are handled in the tests. So we
@@ -455,21 +476,21 @@ def validate_email_deliverability(domain, domain_i18n, timeout=DEFAULT_TIMEOUT):
 
         try:
             # Try resolving for MX records and get them in sorted priority order.
-            response = dns.resolver.query(domain, "MX")
+            response = dns_resolver_resolve_shim(resolver, domain, "MX")
             mtas = sorted([(r.preference, str(r.exchange).rstrip('.')) for r in response])
             mx_fallback = None
         except (dns.resolver.NoNameservers, dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
 
             # If there was no MX record, fall back to an A record.
             try:
-                response = dns.resolver.query(domain, "A")
+                response = dns_resolver_resolve_shim(resolver, domain, "A")
                 mtas = [(0, str(r)) for r in response]
                 mx_fallback = "A"
             except (dns.resolver.NoNameservers, dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
 
                 # If there was no A record, fall back to an AAAA record.
                 try:
-                    response = dns.resolver.query(domain, "AAAA")
+                    response = dns_resolver_resolve_shim(resolver, domain, "AAAA")
                     mtas = [(0, str(r)) for r in response]
                     mx_fallback = "AAAA"
                 except (dns.resolver.NoNameservers, dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
@@ -504,32 +525,31 @@ def main():
     import sys
     import json
 
+    def __utf8_input_shim(input_str):
+        if sys.version_info < (3,):
+            return input_str.decode("utf-8")
+        return input_str
+
+    def __utf8_output_shim(output_str):
+        if sys.version_info < (3,):
+            return unicode_class(output_str).encode("utf-8")
+        return output_str
+
     if len(sys.argv) == 1:
-        # Read lines for STDIN and validate the email address on each line.
-        allow_smtputf8 = True
         for line in sys.stdin:
+            email = __utf8_input_shim(line.strip())
             try:
-                email = line.strip()
-                if sys.version_info < (3,):
-                    email = email.decode("utf8")  # assume utf8 in input
-                validate_email(email, allow_smtputf8=allow_smtputf8)
+                validate_email(email)
             except EmailNotValidError as e:
-                print(email, e)
+                print(__utf8_output_shim("{} {}".format(email, e)))
     else:
         # Validate the email address passed on the command line.
-        email = sys.argv[1]
-        allow_smtputf8 = True
-        check_deliverability = True
-        if sys.version_info < (3,):
-            email = email.decode("utf8")  # assume utf8 in input
+        email = __utf8_input_shim(sys.argv[1])
         try:
-            result = validate_email(email, allow_smtputf8=allow_smtputf8, check_deliverability=check_deliverability)
-            print(json.dumps(result, indent=2, sort_keys=True, ensure_ascii=False))
+            result = validate_email(email)
+            print(json.dumps(result.as_dict(), indent=2, sort_keys=True, ensure_ascii=False))
         except EmailNotValidError as e:
-            if sys.version_info < (3,):
-                print(unicode_class(e).encode("utf8"))
-            else:
-                print(e)
+            print(__utf8_output_shim(e))
 
 
 if __name__ == "__main__":
